from django.db import models
from django.urls import reverse
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType

from tickets.models import Ticket


class Company(models.Model):
    legal_name = models.CharField(max_length=150, help_text="Настоящее имя юридического лица")

    # FIXME: Нет уверенности, что после удаления пользователя следует оставлять `models.DO_NOTHING`
    repr_id =models.ForeignKey(get_user_model(), blank=True, on_delete=models.DO_NOTHING, help_text="Уникальный идентификатор представителя компании")
    
    requirements = models.CharField(max_length=150, default="DS01y04y05y06y07yMPUCS13MPUB", help_text="Код необходимый для того, чтобы ресурс мог стать активным")
    status = models.CharField(max_length=150, default="deactivated", help_text="Cтатус обработки заявки")

    # FIXME: Нет уверенности, что после удаления заявки следует оставлять `models.DO_NOTHING`
    ticket_status = models.ForeignKey(Ticket, on_delete=models.SET_NULL, blank=True, null=True, help_text="ID заявки, привязанной к текущей записи")

    inn = models.CharField(max_length=10, unique=True, help_text="ИНН компании")
    kpp = models.CharField(max_length=9, help_text="КПП компании")
    ogrn = models.CharField(max_length=13, help_text="ОГРН компании")
    bank_name = models.CharField(max_length=150, help_text="Название банка с расчетным счетом")
    bik = models.CharField(max_length=9, help_text="БИК компании")
    rs = models.CharField(max_length=50, help_text="Расчетный счет")
    ks = models.CharField(max_length=50, help_text="Корреспондентский счет (счет, открываемый банковской организацией в подразделении самого банка)")
    address = models.CharField(max_length=150, help_text="Юридический адрес")
    is_global = models.CharField(max_length=1, default="f", help_text="Доступно ли для хранения в глобальном сервисе / Нужна синхронизация данных")
    is_visible = models.CharField(max_length=1, default="f", help_text="Доступно ли для планирования в цепочке с другими услугами в глобальном сервисе")
    
    # FIXME: id_metaservice должно принимать текущий идентификатор метасервиса, на котором происходит создание записи
    id_metaservice = models.PositiveBigIntegerField(default=1, help_text="Уникальный идентификатор метасервиса, необходимый для синхронизации данных. Один и тот же провайдер может быть для нескольких метасервисов, соответственно если происходят изменения в одном, то либо форсируется изменение во всех (если возможно), либо снимается is_global. Соответственно при изменении is_global в true должно происходить согласование с остальными копиями в других сервисах. Нужен в том числе для того, чтобы выяснять, в каких еще сервисах есть этот провайдер.")
    
    def get_admin_url(self):
        """
        Возвращает URL-адрес текущего объекта в административной панели Django.

        Returns:
            str: URL-адрес текущего объекта в административной панели Django
        """
        content_type = ContentType.objects.get_for_model(self.__class__)
        return reverse("admin:%s_%s_change" % (content_type.app_label, content_type.model), args=(self.id,))

    def deactivate(self):
        self.status = "deactivated"
        self.save()

    class Meta:
        db_table = "company"
        verbose_name = "Компания"
        verbose_name_plural = "Компании"